@@ -9,7 +9,7 @@
 #
 package Test::Moose::More;
 {
-  $Test::Moose::More::VERSION = '0.011';
+  $Test::Moose::More::VERSION = '0.015';
 }
 
 # ABSTRACT: More tools for testing Moose packages
@@ -23,10 +23,12 @@ use Sub::Exporter -setup => {
         has_method_ok
         requires_method_ok
         check_sugar_ok check_sugar_removed_ok
+        has_attribute_ok
+        attribute_options_ok
+        validate_attribute
         validate_class validate_role
         meta_ok does_ok does_not_ok
         with_immutable
-        has_attribute_ok
     } ],
     groups  => { default => [ ':all' ] },
 };
@@ -35,7 +37,10 @@ use Test::Builder;
 use Test::More;
 use Test::Moose 'with_immutable';
 use Scalar::Util 'blessed';
+use Perl6::Junction 'any';
+use Moose::Autobox;
 use Moose::Util 'does_role', 'find_meta';
+use Moose::Util::TypeConstraints;
 use Data::OptList;
 
 # debugging...
@@ -218,8 +223,10 @@ sub validate_thing {
                 skip 'Cannot examine attribute metaclass in roles', 1
                     if (find_meta($thing)->isa('Moose::Meta::Role'));
 
-                    local $THING_NAME = "${thing}'s attribute $name";
-                    validate_thing(find_meta($thing)->get_attribute($name), %$opts);
+                local $THING_NAME = "${thing}'s attribute $name";
+                $tb->subtest("[subtest] checking $THING_NAME" => sub {
+                    _validate_attribute(find_meta($thing)->get_attribute($name), %$opts);
+                });
             }
         }
     }
@@ -251,9 +258,110 @@ sub validate_role {
     return validate_thing $role => %args;
 }
 
+
+
+sub validate_attribute {
+    my ($thing, $name, %opts) = @_;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    has_attribute_ok($thing, $name);
+    my $att = find_meta($thing)->get_attribute($name)
+        or return;
+
+    return _validate_attribute($att, %opts);
+}
+
+sub _validate_attribute {
+    my ($att, %opts) = @_;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my %thing_opts =
+        map  { $_ => delete $opts{"-$_"} }
+        map  { s/^-//; $_                }
+        grep { /^-/                      }
+        sort keys %opts
+        ;
+
+    validate_thing $att => %thing_opts
+        if keys %thing_opts;
+
+    return _attribute_options_ok($att, %opts);
+}
+
+sub attribute_options_ok {
+    my ($thing, $name, %opts) = @_;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    has_attribute_ok($thing, $name);
+    my $att = find_meta($thing)->get_attribute($name)
+        or return;
+
+    return _attribute_options_ok($att, %opts);
+}
+
+sub _attribute_options_ok {
+    my ($att, %opts) = @_;
+
+    my @check_opts =
+        qw{ reader writer accessor predicate default builder clearer };
+    my @unhandled_opts = qw{ isa does handles traits };
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my $name = $att->name;
+
+    my $thing_name = _thing_name($name, $att);
+
+    # XXX do we really want to do this?
+    if (my $is = delete $opts{is}) {
+        $opts{accessor} = $name if $is eq 'rw' && ! exists $opts{accessor};
+        $opts{reader}   = $name if $is eq 'ro' && ! exists $opts{reader};
+    }
+
+    my $check = sub {
+        my $property = shift || $_;
+        my $value    = delete $opts{$property};
+        my $has      = "has_$property";
+
+        defined $value
+            ? ok($att->$has,  "$thing_name has a $property")
+            : ok(!$att->$has, "$thing_name does not have a $property")
+            ;
+        is($att->$property, $value, "$thing_name option $property correct")
+    };
+
+    $check->($_) for grep { any(@check_opts) eq $_ } sort keys %opts;
+
+    do { $tb->skip("cannot test '$_' options yet", 1); delete $opts{$_} }
+        for grep { exists $opts{$_} } @unhandled_opts;
+
+    if (exists $opts{init_arg}) {
+
+        $opts{init_arg}
+            ?  $check->('init_arg')
+            : ok(!$att->has_init_arg, "$thing_name has no init_arg")
+            ;
+        delete $opts{init_arg};
+    }
+
+    if (exists $opts{lazy}) {
+
+        delete $opts{lazy}
+            ? ok($att->is_lazy,  "$thing_name is lazy")
+            : ok(!$att->is_lazy, "$thing_name is not lazy")
+            ;
+    }
+
+    fail "unknown attribute option: $_"
+        for sort keys %opts;
+
+    return;
+}
+
+1;
+
 !!42;
 
-
+__END__
 
 =pod
 
@@ -267,7 +375,7 @@ Test::Moose::More - More tools for testing Moose packages
 
 =head1 VERSION
 
-This document describes version 0.011 of Test::Moose::More - released August 26, 2012 as part of Test-Moose-More.
+This document describes version 0.015 of Test::Moose::More - released October 20, 2012 as part of Test-Moose-More.
 
 =head1 SYNOPSIS
 
@@ -383,6 +491,53 @@ additional role-specific tests.
 The same as validate_thing(), but ensures C<$thing> is a class, and allows for
 additional class-specific tests.
 
+=head2 validate_attribute
+
+validate_attribute() allows you to test how an attribute looks once built and
+attached to a class.
+
+Let's say you have an attribute defined like this:
+
+    has foo => (
+        traits  => [ 'TestRole' ],
+        is      => 'ro',
+        isa     => 'Int',
+        builder => '_build_foo',
+        lazy    => 1,
+    );
+
+You can use validate_attribute() to ensure that it's built out in the way you
+expect:
+
+    validate_attribute TestClass => foo => (
+        -does => [ 'TestRole' ],
+        -isa  => [ 'Moose::Meta::Attribute' ], # for demonstration's sake
+        traits   => [ 'TestRole' ],
+        isa      => 'Int',
+        does     => 'Bar',
+        handles  => { },
+        reader   => 'foo',
+        builder  => '_build_foo',
+        default  => undef,
+        init_arg => 'foo',
+        lazy     => 1,
+    );
+
+Not yet documented or tested exhaustively; please see t/validate_attribute.t
+for details at the moment.  This test routine is likely to change in
+implementation and scope, with every effort to maintain backwards
+compatibility.
+
+=head2 attribute_options_ok
+
+Validates that an attribute is set up as expected; like validate_attribute(),
+but only concerns itself with attribute options.
+
+Not yet documented or tested exhaustively; please see t/validate_attribute.t
+for details at the moment.  This test routine is likely to change in
+implementation and scope, with every effort to maintain backwards
+compatibility.
+
 =head1 SEE ALSO
 
 Please see those modules/websites for more information related to this module.
@@ -397,13 +552,13 @@ L<Test::Moose>
 
 =head1 SOURCE
 
-The development version is on github at L<http://github.com/RsrchBoy/test-moose-more>
-and may be cloned from L<git://github.com/RsrchBoy/test-moose-more.git>
+The development version is on github at L<http://github.com/RsrchBoy/Test-Moose-More>
+and may be cloned from L<git://github.com/RsrchBoy/Test-Moose-More.git>
 
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-https://github.com/RsrchBoy/test-moose-more/issues
+https://github.com/RsrchBoy/Test-Moose-More/issues
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
@@ -422,7 +577,3 @@ This is free software, licensed under:
   The GNU Lesser General Public License, Version 2.1, February 1999
 
 =cut
-
-
-__END__
-
